@@ -97,6 +97,14 @@ typedef struct {
 	guint64 last_typeselect_time;
 } TypeSelectState;
 
+enum {
+	LABEL_COLOR,
+	LABEL_COLOR_HIGHLIGHT,
+	LABEL_INFO_COLOR,
+	LABEL_INFO_COLOR_HIGHLIGHT,
+	LAST_LABEL_COLOR
+};
+
 struct NautilusIconContainerDetails {
 	/* List of icons. */
 	GList *icons;
@@ -166,8 +174,10 @@ struct NautilusIconContainerDetails {
 	/* zoom level */
 	int zoom_level;
 
-	/* fonts used to draw labels */
-	char *font_name;
+	/* specific fonts used to draw labels */
+	char *font;
+	
+	/* font sizes used to draw labels */
 	int font_size_table[NAUTILUS_ZOOM_LEVEL_LARGEST + 1];
 
 	/* pixbuf and color for label highlighting */
@@ -175,11 +185,9 @@ struct NautilusIconContainerDetails {
 	GdkColor   highlight_color;
 	guint32    highlight_color_rgba;
 	
-	/* color for text labels */
-	GdkColor label_color;
-	GdkColor label_color_highlight;
-	GdkColor label_info_color;
-	GdkColor label_info_color_highlight;
+	/* colors for text labels */
+	GdkGC   *label_gcs    [LAST_LABEL_COLOR];
+	GdkColor label_colors [LAST_LABEL_COLOR];
 	
 	/* State used so arrow keys don't wander if icons aren't lined up.
 	 * Keeps track of last axis arrow key was used on.
@@ -213,6 +221,10 @@ struct NautilusIconContainerDetails {
 	int right_margin;
 	int top_margin;
 	int bottom_margin;
+
+	/* Whether we should use drop shadows for the icon labels or not */
+	gboolean use_drop_shadows;
+	gboolean drop_shadows_requested;
 };
 
 /* Private functions shared by mutiple files. */
@@ -245,7 +257,8 @@ gboolean      nautilus_icon_container_scroll                      (NautilusIconC
 void          nautilus_icon_container_update_scroll_region        (NautilusIconContainer *container);
 
 /* label color for items */
-GdkColor     *nautilus_icon_container_get_label_color             (NautilusIconContainer *container,
+GdkGC        *nautilus_icon_container_get_label_color_and_gc      (NautilusIconContainer *container,
+								   GdkColor             **color,
 								   gboolean               first_line,
 								   gboolean               needs_highlight);
 
