@@ -38,10 +38,15 @@
 #include <gtk/gtktreeview.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-macros.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
+#include <libnautilus-private/eggtreemultidnd.h>
 #include <libnautilus-private/nautilus-directory-background.h>
 #include <libnautilus-private/nautilus-dnd.h>
+#include <libnautilus-private/nautilus-file-dnd.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
+#include <libnautilus-private/nautilus-icon-dnd.h>
 #include <libnautilus-private/nautilus-metadata.h>
+#include <libnautilus-private/nautilus-tree-view-drag-dest.h>
 
 struct FMListViewDetails {
 	GtkTreeView *tree_view;
@@ -56,12 +61,8 @@ struct FMListViewDetails {
 	GtkCellRendererText   *date_modified_cell;
 
 	NautilusZoomLevel zoom_level;
-};
 
-static GtkTargetEntry drag_types [] = {
-	{ NAUTILUS_ICON_DND_GNOME_ICON_LIST_TYPE, 0, NAUTILUS_ICON_DND_GNOME_ICON_LIST },
-	{ NAUTILUS_ICON_DND_URI_LIST_TYPE, 0, NAUTILUS_ICON_DND_URI_LIST },
-	{ NAUTILUS_ICON_DND_URL_TYPE, 0, NAUTILUS_ICON_DND_URL }
+	NautilusTreeViewDragDest *drag_dest;
 };
 
 /*
@@ -80,6 +81,9 @@ static GList *              fm_list_view_get_selection         (FMDirectoryView
 static void                 fm_list_view_set_zoom_level        (FMListView *view,
 								NautilusZoomLevel new_level,
 								gboolean always_set_level);
+static void		    fm_list_view_scale_font_size       (FMListView *view, 
+								NautilusZoomLevel new_level,
+								gboolean update_size_table);
 
 GNOME_CLASS_BOILERPLATE (FMListView, fm_list_view,
 			 FMDirectoryView, FM_TYPE_DIRECTORY_VIEW)
@@ -137,31 +141,30 @@ event_after_callback (GtkWidget *widget, GdkEventAny *event, gpointer callback_d
 		fm_directory_view_activate_files (view, file_list);
 		nautilus_file_list_free (file_list);
 	}
-
-	if (event->type == GDK_BUTTON_PRESS
-	    && event->window == gtk_tree_view_get_bin_window (GTK_TREE_VIEW (widget))
-	    && (((GdkEventButton *) event)->button == 3)) {
-		/* Put up the right kind of menu if we right click in the tree view. */
-		if (tree_view_has_selection (GTK_TREE_VIEW (widget))) {
-			fm_directory_view_pop_up_selection_context_menu (view, (GdkEventButton *) event);
-		} else {
-			fm_directory_view_pop_up_background_context_menu (view, (GdkEventButton *) event);
-		}
-	}
 }
 
 static gboolean
 button_press_callback (GtkWidget *widget, GdkEventButton *event, gpointer callback_data)
 {
+	FMListView *view;
 	GtkTreeView *tree_view;
 	GtkTreePath *path;
+	gboolean result;
 
+	view = FM_LIST_VIEW (callback_data);
 	tree_view = GTK_TREE_VIEW (widget);
 
 	if (event->window != gtk_tree_view_get_bin_window (tree_view)) {
 		return FALSE;
 	}
 
+	fm_list_model_set_drag_view
+		(FM_LIST_MODEL (gtk_tree_view_get_model (tree_view)),
+		 tree_view,
+		 event->x, event->y);
+
+	result = FALSE;
+
 	if (gtk_tree_view_get_path_at_pos (tree_view, event->x, event->y,
 					   &path, NULL, NULL, NULL)) {
 		if (event->button == 3
@@ -170,7 +173,7 @@ button_press_callback (GtkWidget *widget, GdkEventButton *event, gpointer callba
 			   are selected it will unselect all but one row; but we
 			   want the right click menu to apply to everything that's
 			   currently selected. */
-			return TRUE;
+			result = TRUE;
 		}
 
 		gtk_tree_path_free (path);
@@ -180,7 +183,7 @@ button_press_callback (GtkWidget *widget, GdkEventButton *event, gpointer callba
 		gtk_tree_selection_unselect_all (gtk_tree_view_get_selection (tree_view));
 	}
 
-	return FALSE;
+	return result;
 }
 
 static gboolean
@@ -200,6 +203,16 @@ button_release_callback (GtkWidget *widget, GdkEventButton *event, gpointer call
 		nautilus_file_list_free (file_list);
 	}
 
+	if (event->window == gtk_tree_view_get_bin_window (GTK_TREE_VIEW (widget))
+	    && (event->button == 3)) {
+		/* Put up the right kind of menu if we right click in the tree view. */
+		if (tree_view_has_selection (GTK_TREE_VIEW (widget))) {
+			fm_directory_view_pop_up_selection_context_menu (FM_DIRECTORY_VIEW (view), (GdkEventButton *) event);
+		} else {
+			fm_directory_view_pop_up_background_context_menu (FM_DIRECTORY_VIEW (view), (GdkEventButton *) event);
+		}
+	}
+
 	return FALSE;
 }
 
@@ -281,14 +294,97 @@ cell_renderer_edited (GtkCellRendererText *cell,
 	nautilus_file_unref (file);
 }
 
+static char *
+get_root_uri_callback (NautilusTreeViewDragDest *dest,
+		       gpointer user_data)
+{
+	FMListView *view;
+	
+	view = FM_LIST_VIEW (user_data);
+
+	return fm_directory_view_get_uri (FM_DIRECTORY_VIEW (view));
+}
+
+static NautilusFile *
+get_file_for_path_callback (NautilusTreeViewDragDest *dest,
+			    GtkTreePath *path,
+			    gpointer user_data)
+{
+	FMListView *view;
+	GtkTreeIter iter;
+	NautilusFile *file;
+	
+	view = FM_LIST_VIEW (user_data);
+
+	file = NULL;
+
+	if (gtk_tree_model_get_iter (GTK_TREE_MODEL (view->details->model),
+				     &iter, path)) {
+		gtk_tree_model_get (GTK_TREE_MODEL (view->details->model),
+				    &iter,
+				    FM_LIST_MODEL_FILE_COLUMN,
+				    &file,
+				    -1);
+	}
+
+	return file;
+}
+
+static void
+move_copy_items_callback (NautilusTreeViewDragDest *dest,
+			  const GList *item_uris,
+			  const char *target_uri,
+			  guint action,
+			  int x, 
+			  int y,
+			  gpointer user_data)
+
+{
+	fm_directory_view_move_copy_items (item_uris,
+					   NULL,
+					   target_uri,
+					   action,
+					   x, y,
+					   FM_DIRECTORY_VIEW (user_data));
+}
+
 static void
 create_and_set_up_tree_view (FMListView *view)
 {
 	GtkCellRenderer *cell;
 	GtkTreeViewColumn *column;
-	
+	GtkTargetEntry *drag_types;
+	int num_drag_types;	
+
 	view->details->tree_view = GTK_TREE_VIEW (gtk_tree_view_new ());
 
+	fm_list_model_get_drag_types (&drag_types, &num_drag_types);
+
+	egg_tree_multi_drag_add_drag_support (view->details->tree_view);
+
+	gtk_tree_view_enable_model_drag_source
+		(view->details->tree_view,
+		 GDK_BUTTON1_MASK | GDK_BUTTON3_MASK,
+		 drag_types,
+		 num_drag_types,
+		 GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK | GDK_ACTION_ASK);
+
+	view->details->drag_dest = 
+		nautilus_tree_view_drag_dest_new (view->details->tree_view);
+
+	g_signal_connect_object (view->details->drag_dest,
+				 "get_root_uri",
+				 G_CALLBACK (get_root_uri_callback),
+				 view, 0);
+	g_signal_connect_object (view->details->drag_dest,
+				 "get_file_for_path",
+				 G_CALLBACK (get_file_for_path_callback),
+				 view, 0);
+	g_signal_connect_object (view->details->drag_dest,
+				 "move_copy_items",
+				 G_CALLBACK (move_copy_items_callback),
+				 view, 0);
+
 	g_signal_connect_object (gtk_tree_view_get_selection (view->details->tree_view),
 				 "changed",
 				 G_CALLBACK (list_selection_changed_callback), view, 0);
@@ -311,10 +407,6 @@ create_and_set_up_tree_view (FMListView *view)
 	gtk_tree_selection_set_mode (gtk_tree_view_get_selection (view->details->tree_view), GTK_SELECTION_MULTIPLE);
 	gtk_tree_view_set_rules_hint (view->details->tree_view, TRUE);
 
-	gtk_tree_view_enable_model_drag_source (view->details->tree_view, 0,
-						drag_types, G_N_ELEMENTS (drag_types),
-						GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK);
-
 	/* Create the file name column */
 	cell = gtk_cell_renderer_pixbuf_new ();
 	view->details->pixbuf_cell = (GtkCellRendererPixbuf *)cell;
@@ -325,6 +417,10 @@ create_and_set_up_tree_view (FMListView *view)
 	gtk_tree_view_column_set_resizable (view->details->file_name_column, TRUE);
 
 	gtk_tree_view_column_pack_start (view->details->file_name_column, cell, FALSE);
+	gtk_tree_view_column_set_attributes (view->details->file_name_column,
+					     cell,
+					     "pixbuf", FM_LIST_MODEL_SMALLEST_ICON_COLUMN,
+					     NULL);
 
 	cell = gtk_cell_renderer_text_new ();
 	view->details->file_name_cell = (GtkCellRendererText *)cell;
@@ -411,6 +507,30 @@ set_sort_order_from_metadata_and_preferences (FMListView *list_view)
 					      sort_reversed ? GTK_SORT_DESCENDING : GTK_SORT_ASCENDING);
 }
 
+static gboolean
+list_view_changed_foreach (GtkTreeModel *model,
+              		   GtkTreePath  *path,
+			   GtkTreeIter  *iter,
+			   gpointer      data)
+{
+	gtk_tree_model_row_changed (model, path, iter);
+	return FALSE;
+}
+
+static NautilusZoomLevel
+get_default_zoom_level (void) {
+	NautilusZoomLevel default_zoom_level;
+
+	default_zoom_level = default_zoom_level_auto_value;
+
+	if (default_zoom_level <  NAUTILUS_ZOOM_LEVEL_SMALLEST
+	    || NAUTILUS_ZOOM_LEVEL_LARGEST < default_zoom_level) {
+		default_zoom_level = NAUTILUS_ZOOM_LEVEL_SMALL;
+	}
+
+	return default_zoom_level;
+}
+
 static void
 set_zoom_level_from_metadata_and_preferences (FMListView *list_view)
 {
@@ -421,8 +541,15 @@ set_zoom_level_from_metadata_and_preferences (FMListView *list_view)
 		file = fm_directory_view_get_directory_as_file (FM_DIRECTORY_VIEW (list_view));
 		level = nautilus_file_get_integer_metadata (file,
 							    NAUTILUS_METADATA_KEY_LIST_VIEW_ZOOM_LEVEL, 
-							    default_zoom_level_auto_value);
+							    get_default_zoom_level ());
 		fm_list_view_set_zoom_level (list_view, level, TRUE);
+		
+		/* reset the font size table for the new default zoom level */
+		fm_list_view_scale_font_size (list_view, level, TRUE);
+		
+		/* updated the rows after updating the font size */
+		gtk_tree_model_foreach (GTK_TREE_MODEL (list_view->details->model),
+					list_view_changed_foreach, NULL);
 	}
 }
 
@@ -549,7 +676,47 @@ fm_list_view_reset_to_defaults (FMDirectoryView *view)
 		 fm_list_model_get_sort_column_id_from_sort_type (default_sort_order_auto_value),
 		 default_sort_reversed_auto_value ? GTK_SORT_DESCENDING : GTK_SORT_ASCENDING);
 
-	fm_list_view_set_zoom_level (FM_LIST_VIEW (view), default_zoom_level_auto_value, FALSE);
+	fm_list_view_set_zoom_level (FM_LIST_VIEW (view), get_default_zoom_level (), FALSE);
+}
+
+static void
+fm_list_view_scale_font_size (FMListView *view, 
+			      NautilusZoomLevel new_level,
+			      gboolean update_size_table)
+{
+	static gboolean first_time = TRUE;
+	static double pango_scale[7];
+	int default_zoom_level, i;
+
+	g_return_if_fail (new_level >= NAUTILUS_ZOOM_LEVEL_SMALLEST &&
+			  new_level <= NAUTILUS_ZOOM_LEVEL_LARGEST);
+
+	if (update_size_table || first_time) {
+		first_time = FALSE;
+
+		default_zoom_level = get_default_zoom_level ();
+
+		pango_scale[default_zoom_level] = PANGO_SCALE_MEDIUM;
+		for (i = default_zoom_level; i > NAUTILUS_ZOOM_LEVEL_SMALLEST; i--) {
+			pango_scale[i - 1] = (1 / 1.2) * pango_scale[i];
+		}
+		for (i = default_zoom_level; i < NAUTILUS_ZOOM_LEVEL_LARGEST; i++) {
+			pango_scale[i + 1] = 1.2 * pango_scale[i];
+		}
+	}
+					 
+	g_object_set (G_OBJECT (view->details->file_name_cell),
+		      "scale", pango_scale[new_level],
+		      NULL);
+	g_object_set (G_OBJECT (view->details->size_cell),
+		      "scale", pango_scale[new_level],
+		      NULL);
+	g_object_set (G_OBJECT (view->details->type_cell),
+		      "scale", pango_scale[new_level],
+		      NULL);
+	g_object_set (G_OBJECT (view->details->date_modified_cell),
+		      "scale", pango_scale[new_level],
+		      NULL);
 }
 
 static void
@@ -557,13 +724,6 @@ fm_list_view_set_zoom_level (FMListView *view,
 			     NautilusZoomLevel new_level,
 			     gboolean always_set_level)
 {
-	static double pango_scale[7] = { PANGO_SCALE_X_SMALL,
-					 PANGO_SCALE_SMALL,
-					 PANGO_SCALE_MEDIUM,
-					 PANGO_SCALE_LARGE,
-					 PANGO_SCALE_X_LARGE,
-					 PANGO_SCALE_XX_LARGE,
-					 1.2 * PANGO_SCALE_XX_LARGE };
 	int icon_size;
 	int column;
 
@@ -584,7 +744,7 @@ fm_list_view_set_zoom_level (FMListView *view,
 	nautilus_file_set_integer_metadata
 		(fm_directory_view_get_directory_as_file (FM_DIRECTORY_VIEW (view)), 
 		 NAUTILUS_METADATA_KEY_LIST_VIEW_ZOOM_LEVEL, 
-		 default_zoom_level_auto_value,
+		 get_default_zoom_level (),
 		 new_level);
 
 	/* Select correctly scaled icons. */
@@ -595,18 +755,7 @@ fm_list_view_set_zoom_level (FMListView *view,
 					     NULL);
 
 	/* Scale text. */
-	g_object_set (G_OBJECT (view->details->file_name_cell),
-		      "scale", pango_scale[new_level],
-		      NULL);
-	g_object_set (G_OBJECT (view->details->size_cell),
-		      "scale", pango_scale[new_level],
-		      NULL);
-	g_object_set (G_OBJECT (view->details->type_cell),
-		      "scale", pango_scale[new_level],
-		      NULL);
-	g_object_set (G_OBJECT (view->details->date_modified_cell),
-		      "scale", pango_scale[new_level],
-		      NULL);
+	fm_list_view_scale_font_size (view, new_level, FALSE);
 
 	/* Make all rows the same size. */
 	icon_size = nautilus_get_icon_size_for_zoom_level (new_level);
@@ -654,7 +803,7 @@ fm_list_view_restore_default_zoom_level (FMDirectoryView *view)
 
 	list_view = FM_LIST_VIEW (view);
 
-	fm_list_view_set_zoom_level (list_view, default_zoom_level_auto_value, FALSE);
+	fm_list_view_set_zoom_level (list_view, get_default_zoom_level (), FALSE);
 }
 
 static gboolean 
@@ -710,6 +859,9 @@ click_policy_changed_callback (gpointer callback_data)
 			      "underline", PANGO_UNDERLINE_NONE,
 			      NULL);
 	}
+
+	gtk_tree_model_foreach (GTK_TREE_MODEL (view->details->model),
+				list_view_changed_foreach, NULL);
 }
 
 static void
@@ -755,6 +907,11 @@ fm_list_view_dispose (GObject *object)
 		list_view->details->model = NULL;
 	}
 
+	if (list_view->details->drag_dest) {
+		g_object_unref (list_view->details->drag_dest);
+		list_view->details->drag_dest = NULL;
+	}
+
 	G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
@@ -843,6 +1000,6 @@ fm_list_view_instance_init (FMListView *list_view)
 						  list_view, G_OBJECT (list_view));
 
 	click_policy_changed_callback (list_view);
-
+	
 	fm_list_view_sort_directories_first_changed (FM_DIRECTORY_VIEW (list_view));
 }
